/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer, Aina Niemetz, Morgan Deters
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2024 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Contains code for handling command-line options.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.h file.
 */

#include "cvc5_private.h"

#ifndef CVC5__OPTIONS__UF_H
#define CVC5__OPTIONS__UF_H

#include "options/options.h"

// clang-format off

// clang-format on

namespace cvc5::internal::options {

namespace uf::longName {
// clang-format off
  static constexpr const char* eagerArithBvConv = "eager-arith-bv-conv";
    static constexpr const char* modelBasedArithBvConv = "model-based-arith-bv-conv";
    static constexpr const char* ufSymmetryBreaker = "symmetry-breaker";
    static constexpr const char* ufHoExt = "uf-ho-ext";
    static constexpr const char* ufHoLambdaQe = "uf-lambda-qe";
    static constexpr const char* ufHoLazyLambdaLift = "uf-lazy-ll";
    static constexpr const char* ufssMode = "uf-ss";
    static constexpr const char* ufssAbortCardinality = "uf-ss-abort-card";
    static constexpr const char* ufssFairness = "uf-ss-fair";
    static constexpr const char* ufssFairnessMonotone = "uf-ss-fair-monotone";
// clang-format on
}

// clang-format off
enum class UfssMode
{
  FULL, NO_MINIMAL, NONE,
  __MAX_VALUE = NONE
};
std::ostream& operator<<(std::ostream& os, UfssMode mode);
UfssMode stringToUfssMode(const std::string& optarg);

// clang-format on

#if defined(CVC5_MUZZLED) || defined(CVC5_COMPETITION_MODE)
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT false
#else /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT true
#endif /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */

struct HolderUF
{
// clang-format off
  bool eagerArithBvConv = false;
  bool eagerArithBvConvWasSetByUser = false;
  bool modelBasedArithBvConv = false;
  bool modelBasedArithBvConvWasSetByUser = false;
  bool ufSymmetryBreaker = false;
  bool ufSymmetryBreakerWasSetByUser = false;
  bool ufHoExt = true;
  bool ufHoExtWasSetByUser = false;
  bool ufHoLambdaQe = false;
  bool ufHoLambdaQeWasSetByUser = false;
  bool ufHoLazyLambdaLift = true;
  bool ufHoLazyLambdaLiftWasSetByUser = false;
  UfssMode ufssMode = UfssMode::FULL;
  bool ufssModeWasSetByUser = false;
  int64_t ufssAbortCardinality = -1;
  bool ufssAbortCardinalityWasSetByUser = false;
  bool ufssFairness = true;
  bool ufssFairnessWasSetByUser = false;
  bool ufssFairnessMonotone = false;
  bool ufssFairnessMonotoneWasSetByUser = false;
// clang-format on
};

#undef DO_SEMANTIC_CHECKS_BY_DEFAULT


}  // namespace cvc5::internal::options

#endif /* CVC5__OPTIONS__UF_H */
